@@ -31,6 +31,7 @@ from urllib3.connectionpool import HTTPConnectionPool
 from urllib3.exceptions import (ConnectTimeoutError,
                                 MaxRetryError,
                                 ReadTimeoutError)
+from warnings import warn
 from . import _utils, __version__
 from ._models import CdxRecord, Memento
 from .exceptions import (WaybackException,
@@ -45,7 +46,7 @@ from .exceptions import (WaybackException,
 
 logger = logging.getLogger(__name__)
 
-CDX_SEARCH_URL = 'http://web.archive.org/cdx/search/cdx'
+CDX_SEARCH_URL = 'https://web.archive.org/cdx/search/cdx'
 # This /web/timemap URL has newer features, but has other bugs and doesn't
 # support some features, like resume keys (for paging). It ignores robots.txt,
 # while /cdx/search obeys robots.txt (for now). It also has different/extra
@@ -53,9 +54,9 @@ CDX_SEARCH_URL = 'http://web.archive.org/cdx/search/cdx'
 # https://github.com/internetarchive/wayback/blob/bd205b9b26664a6e2ea3c0c2a8948f0dc6ff4519/wayback-cdx-server/src/main/java/org/archive/cdxserver/format/CDX11Format.java#L13-L17  # noqa
 # NOTE: the `length` and `robotflags` fields appear to always be empty
 # TODO: support new/upcoming CDX API
-# CDX_SEARCH_URL = 'http://web.archive.org/web/timemap/cdx'
+# CDX_SEARCH_URL = 'https://web.archive.org/web/timemap/cdx'
 
-ARCHIVE_URL_TEMPLATE = 'http://web.archive.org/web/{timestamp}{mode}/{url}'
+ARCHIVE_URL_TEMPLATE = 'https://web.archive.org/web/{timestamp}{mode}/{url}'
 REDUNDANT_HTTP_PORT = re.compile(r'^(http://[^:/]+):80(.*)$')
 REDUNDANT_HTTPS_PORT = re.compile(r'^(https://[^:/]+):443(.*)$')
 DATA_URL_START = re.compile(r'data:[\w]+/[\w]+;base64')
@@ -77,12 +78,12 @@ class Mode(Enum):
     these values to determine how the response body should be formatted.
 
     For more details, see:
-    http://archive-access.sourceforge.net/projects/wayback/administrator_manual.html#Archival_URL_Replay_Mode
+    https://archive-access.sourceforge.net/projects/wayback/administrator_manual.html#Archival_URL_Replay_Mode
 
     Examples
     --------
     >>> waybackClient.get_memento('https://noaa.gov/',
-    >>>                           datetime=datetime.datetime(2018, 1, 2),
+    >>>                           timestamp=datetime.datetime(2018, 1, 2),
     >>>                           mode=wayback.Mode.view)
 
     **Values**
@@ -187,12 +188,9 @@ HTTPConnectionPool.ResponseCls = WaybackResponse
 #####################################################################
 
 
-# TODO: make rate limiting configurable at the session level, rather than
-# arbitrarily set inside get_memento(). Idea: have a rate limit lock type and
-# pass an instance to the constructor here.
 class WaybackSession(_utils.DisableAfterCloseSession, requests.Session):
     """
-    A custom session object that network pools connections and resources for
+    A custom session object that pools network connections and resources for
     requests to the Wayback Machine.
 
     Parameters
@@ -211,10 +209,16 @@ class WaybackSession(_utils.DisableAfterCloseSession, requests.Session):
     timeout : int or float or tuple of (int or float, int or float), default: 60
         A timeout to use for all requests.
         See the Requests docs for more:
-        http://docs.python-requests.org/en/master/user/advanced/#timeouts
+        https://docs.python-requests.org/en/master/user/advanced/#timeouts
     user_agent : str, optional
         A custom user-agent string to use in all requests. Defaults to:
         `wayback/{version} (+https://github.com/edgi-govdata-archiving/wayback)`
+    search_calls_per_second : int or float, default: 1.5
+        The maximum number of calls made to the search API per second.
+        To disable the rate limit, set this to 0.
+    memento_calls_per_second : int or float, default: 30
+        The maximum number of calls made to the memento API per second.
+        To disable the rate limit, set this to 0.
     """
 
     # It seems Wayback sometimes produces 500 errors for transient issues, so
@@ -227,7 +231,8 @@ class WaybackSession(_utils.DisableAfterCloseSession, requests.Session):
     # just the error type. See `should_retry_error()`.
     handleable_errors = (ConnectionError,) + retryable_errors
 
-    def __init__(self, retries=6, backoff=2, timeout=60, user_agent=None):
+    def __init__(self, retries=6, backoff=2, timeout=60, user_agent=None,
+                 search_calls_per_second=1.5, memento_calls_per_second=30):
         super().__init__()
         self.retries = retries
         self.backoff = backoff
@@ -237,6 +242,8 @@ class WaybackSession(_utils.DisableAfterCloseSession, requests.Session):
                            f'wayback/{__version__} (+https://github.com/edgi-govdata-archiving/wayback)'),
             'Accept-Encoding': 'gzip, deflate'
         }
+        self.search_calls_per_second = search_calls_per_second
+        self.memento_calls_per_second = memento_calls_per_second
         # NOTE: the nice way to accomplish retry/backoff is with a urllib3:
         #     adapter = requests.adapters.HTTPAdapter(
         #         max_retries=Retry(total=5, backoff_factor=2,
@@ -360,49 +367,100 @@ class WaybackClient(_utils.DepthCountedContext):
         "Close the client's session."
         self.session.close()
 
-    def search(self, url, *, matchType=None, limit=None, offset=None,
-               fastLatest=None, gzip=None, from_date=None, to_date=None,
-               filter_field=None, collapse=None, showResumeKey=True,
-               resumeKey=None, page=None, pageSize=None, resolveRevisits=True,
-               skip_malformed_results=True, previous_result=None, **kwargs):
+    def search(self, url, *, match_type=None, limit=1000, offset=None,
+               fast_latest=None, from_date=None, to_date=None,
+               filter_field=None, collapse=None, resolve_revisits=True,
+               skip_malformed_results=True,
+               # Deprecated Parameters
+               matchType=None, fastLatest=None, resolveRevisits=None):
         """
-        Search archive.org's CDX API for all captures of a given URL.
+        Search archive.org's CDX API for all captures of a given URL. This
+        returns an iterator of :class:`CdxRecord` objects. The `StopIteration`
+        value is the total count of found captures.
 
-        This will automatically page through all results for a given search.
+        Results include captures with similar, but not exactly matching URLs.
+        They are matched by a SURT-formatted, canonicalized URL that:
 
-        Returns an iterator of CdxRecord objects. The StopIteration value is
-        the total count of found captures.
-
-        Note that even URLs without wildcards may return results with different
-        URLs. Search results are matched by url_key, which is a SURT-formatted,
-        canonicalized URL:
-
-        * Does not differentiate between HTTP and HTTPS
-        * Is not case-sensitive
+        * Does not differentiate between HTTP and HTTPS,
+        * Is not case-sensitive, and
         * Treats ``www.`` and ``www*.`` subdomains the same as no subdomain at
-          all
+          all.
 
-        Note not all CDX API parameters are supported. In particular, this does
-        not support: `output`, `fl`, `showDupeCount`, `showSkipCount`,
-        `lastSkipTimestamp`, `showNumPages`, `showPagedIndex`.
+        This will automatically page through all results for a given search. If
+        you want fewer results, you can stop iterating early:
+
+        .. code-block:: python
+
+          from itertools import islice
+          first10 = list(islice(client.search(...), 10))
 
         Parameters
         ----------
         url : str
-            The URL to query for captures of.
-        matchType : str, optional
-            Must be one of 'exact', 'prefix', 'host', or 'domain'. The default
-            value is calculated based on the format of `url`.
-        limit : int, optional
-            Maximum number of results per page (this iterator will continue to
-            move through all pages unless `showResumeKey=False`, though).
+            The URL to search for captures of.
+
+            Special patterns in ``url`` imply a value for the ``match_type``
+            parameter and match multiple URLs:
+
+            * If the URL starts with `*.` (e.g. ``*.epa.gov``) OR
+              ``match_type='domain'``, the search will include all URLs at the
+              given domain and its subdomains.
+            * If the URL ends with `/*` (e.g. ``https://epa.gov/*``) OR
+              ``match_type='prefix'``, the search will include all URLs that
+              start with the text up to the ``*``.
+            * Otherwise, this returns matches just for the requeted URL.
+
+        match_type : str, optional
+            Determines how to interpret the ``url`` parameter. It must be one of
+            the following:
+
+            * ``exact`` (default) returns results matching the requested URL
+              (see notes about SURT above; this is not an exact string match of
+              the URL you pass in).
+            * ``prefix`` returns results that start with the requested URL.
+            * ``host`` returns results from all URLs at the host in the
+              requested URL.
+            * ``domain`` returns results from all URLs at the domain or any
+              subdomain of the requested URL.
+
+            The default value is calculated based on the format of ``url``.
+
+        limit : int, default: 1000
+            Maximum number of results per request to the API (not the maximum
+            number of results this function yields).
+
+            Negative values return the most recent N results.
+
+            Positive values are complicated! The search server will only scan so
+            much data on each query, and if it finds fewer than ``limit``
+            results before hitting its own internal limits, it will behave as if
+            if there are no more results, even though there may be.
+
+            Unfortunately, ideal values for ``limit`` aren't very predicatable
+            because the search server combines data from different sources, and
+            they do not all behave the same. Their parameters may also be
+            changed over time.
+
+            In general…
+
+            * The default value should work well in typical cases.
+            * For frequently captured URLs, you may want to set a higher value
+              (e.g. 12,000) for more efficient querying.
+            * For infrequently captured URLs, you may want to set a lower value
+              (e.g. 100 or even 10) to ensure that your query does not hit
+              internal limits before returning.
+            * For extremely infrequently captured URLs, you may simply want to
+              call ``search()`` multiple times with different, close together
+              ``from_date`` and ``to_date`` values.
+
         offset : int, optional
             Skip the first N results.
-        fastLatest : bool, optional
-            Get faster results when using a negative value for `limit`. It may
-            return a variable number of results.
-        gzip : bool, optional
-            Whether output should be gzipped.
+        fast_latest : bool, optional
+            Get faster results when using a negative value for ``limit``. It may
+            return a variable number of results that doesn't match the value
+            of ``limit``. For example,
+            ``search('http://epa.gov', limit=-10, fast_latest=True)`` may return
+            any number of results between 1 and 10.
         from_date : datetime or date, optional
             Only include captures after this date. Equivalent to the
             `from` argument in the CDX API. If it does not have a time zone, it
@@ -412,174 +470,182 @@ class WaybackClient(_utils.DepthCountedContext):
             argument in the CDX API. If it does not have a time zone, it is
             assumed to be in UTC.
         filter_field : str, optional
-            A filter for any field in the results. Equivalent to the `filter`
-            argument in the CDX API. (format: `[!]field:regex`)
+            A filter for any field in the results. Equivalent to the ``filter``
+            argument in the CDX API. (format: ``[!]field:regex``)
         collapse : str, optional
             Collapse consecutive results that match on a given field. (format:
             `fieldname` or `fieldname:N` -- N is the number of chars to match.)
-        showResumeKey : bool, optional
-            If False, don't continue to iterate through all pages of results.
-            The default value is True
-        resumeKey : str, optional
-            Start returning results from a specified resumption point/offset.
-            The value for this is supplied by the previous page of results when
-            `showResumeKey` is True.
-        page : int, optional
-            If using paging start from this page number (note: paging, as
-            opposed to the using `resumeKey` is somewhat complicated because
-            of the interplay with indexes and index sizes).
-        pageSize : int, optional
-            The number of index blocks to examine for each page of results.
-            Index blocks generally cover about 3,000 items, so setting
-            `pageSize=1` might return anywhere from 0 to 3,000 results per page.
-        resolveRevists : bool, optional
-            Attempt to resolve `warc/revisit` records to their actual content
-            type and response code. Not supported on all CDX servers. Defaults
-            to True.
-        skip_malformed_results : bool, optional
+        resolve_revisits : bool, default: True
+            Attempt to resolve ``warc/revisit`` records to their actual content
+            type and response code. Not supported on all CDX servers.
+        skip_malformed_results : bool, default: True
             If true, don't yield records that look like they have no actual
             memento associated with them. Some crawlers will erroneously
-            attempt to capture bad URLs like `http://mailto:someone@domain.com`
-            or `http://data:image/jpeg;base64,AF34...` and so on. This is a
+            attempt to capture bad URLs like
+            ``http://mailto:someone@domain.com`` or
+            ``http://data:image/jpeg;base64,AF34...`` and so on. This is a
             filter performed client side and is not a CDX API argument.
-            (Default: True)
-        previous_result : str, optional
-            *For internal use.* The CDX API sometimes returns repeated results.
-            This is used to track the previous result so we can filter out the
-            repeats.
-        **kwargs
-            Any additional CDX API options.
 
         Raises
         ------
         UnexpectedResponseFormat
             If the CDX response was not parseable.
 
-        References
-        ----------
-        * https://github.com/internetarchive/wayback/tree/master/wayback-cdx-server
-
         Yields
         ------
         version: CdxRecord
             A :class:`CdxRecord` encapsulating one capture or revisit
+
+        References
+        ----------
+        * HTTP API Docs: https://github.com/internetarchive/wayback/tree/master/wayback-cdx-server
+        * SURT formatting: http://crawler.archive.org/articles/user_manual/glossary.html#surt
+        * SURT implementation: https://github.com/internetarchive/surt
+
+        Notes
+        -----
+        Several CDX API parameters are not relevant or handled automatically
+        by this function. This does not support: `output`, `fl`,
+        `showDupeCount`, `showSkipCount`, `lastSkipTimestamp`, `showNumPages`,
+        `showPagedIndex`.
+
+        It also does not support `page` and `pageSize` for
+        pagination because they work differently from the `resumeKey` method
+        this uses, and results do not include recent captures when using them.
         """
+        if matchType is not None:
+            warn('The `matchType` parameter for search() was renamed to '
+                 '`match_type`. Support for the old name will be removed in '
+                 'wayback v0.5.0; please update your code.',
+                 DeprecationWarning,
+                 stacklevel=2)
+            match_type = match_type or matchType
+        if fastLatest is not None:
+            warn('The `fastLatest` parameter for search() was renamed to '
+                 '`fast_latest`. Support for the old name will be removed in '
+                 'wayback v0.5.0; please update your code.',
+                 DeprecationWarning,
+                 stacklevel=2)
+            fast_latest = fast_latest or fastLatest
+        if resolveRevisits is not None:
+            warn('The `resolveRevisits` parameter for search() was renamed to '
+                 '`resolve_revisits`. Support for the old name will be removed '
+                 'in wayback v0.5.0; please update your code.',
+                 DeprecationWarning,
+                 stacklevel=2)
+            resolve_revisits = resolve_revisits or resolveRevisits
 
         # TODO: support args that can be set multiple times: filter, collapse
         # Should take input as a sequence and convert to repeat query args
-        # TODO: support args that add new fields to the results or change the
-        # result format
-        query = {'url': url, 'matchType': matchType, 'limit': limit,
-                 'offset': offset, 'gzip': gzip, 'from': from_date,
-                 'to': to_date, 'filter': filter_field,
-                 'fastLatest': fastLatest, 'collapse': collapse,
-                 'showResumeKey': showResumeKey, 'resumeKey': resumeKey,
-                 'resolveRevisits': resolveRevisits, 'page': page,
-                 'pageSize': page}
-        query.update(kwargs)
-
-        unsupported = {'output', 'fl', 'showDupeCount', 'showSkipCount',
-                       'lastSkipTimestamp', 'showNumPages', 'showPagedIndex'}
-
-        final_query = {}
-        for key, value in query.items():
-            if key in unsupported:
-                raise ValueError(f'The {key} argument is not supported')
+        # TODO: Check types
+        query_args = {'url': url, 'matchType': match_type, 'limit': limit,
+                      'offset': offset, 'from': from_date,
+                      'to': to_date, 'filter': filter_field,
+                      'fastLatest': fast_latest, 'collapse': collapse,
+                      'showResumeKey': True,
+                      'resolveRevisits': resolve_revisits}
 
+        query = {}
+        for key, value in query_args.items():
             if value is not None:
                 if isinstance(value, str):
-                    final_query[key] = value
+                    query[key] = value
                 elif isinstance(value, date):
-                    final_query[key] = _utils.format_timestamp(value)
+                    query[key] = _utils.format_timestamp(value)
                 else:
-                    final_query[key] = str(value).lower()
+                    query[key] = str(value).lower()
 
-        response = self.session.request('GET', CDX_SEARCH_URL,
-                                        params=final_query)
-        try:
-            # Read/cache the response and close straightaway. If we need to
-            # raise for status, we want to pre-emptively close the response
-            # so a user handling the error doesn't need to worry about it. If
-            # we don't raise here, we still want to close the connection so it
-            # doesn't leak when we move onto the next of results or when this
-            # iterator ends.
-            read_and_close(response)
-            response.raise_for_status()
-        except requests.exceptions.HTTPError as error:
-            if 'AdministrativeAccessControlException' in response.text:
-                raise BlockedSiteError(query['url'])
-            elif 'RobotAccessControlException' in response.text:
-                raise BlockedByRobotsError(query['url'])
-            else:
-                raise WaybackException(str(error))
-
-        lines = iter(response.content.splitlines())
+        next_query = query
         count = 0
+        previous_result = None
+        while next_query:
+            sent_query, next_query = next_query, None
+            with _utils.rate_limited(self.session.search_calls_per_second,
+                                     group='search'):
+                response = self.session.request('GET', CDX_SEARCH_URL,
+                                                params=sent_query)
+                try:
+                    # Read/cache the response and close straightaway. If we need
+                    # to raise for status, we want to pre-emptively close the
+                    # response so a user handling the error doesn't need to
+                    # worry about it. If we don't raise here, we still want to
+                    # close the connection so it doesn't leak when we move onto
+                    # the next of results or when this iterator ends.
+                    read_and_close(response)
+                    response.raise_for_status()
+                except requests.exceptions.HTTPError as error:
+                    if 'AdministrativeAccessControlException' in response.text:
+                        raise BlockedSiteError(query['url'])
+                    elif 'RobotAccessControlException' in response.text:
+                        raise BlockedByRobotsError(query['url'])
+                    else:
+                        raise WaybackException(str(error))
 
-        for line in lines:
-            text = line.decode()
+            lines = iter(response.content.splitlines())
 
-            # The resume key is delineated by a blank line.
-            if text == '':
-                next_args = query.copy()
-                next_args['resumeKey'] = next(lines).decode()
-                count += yield from self.search(previous_result=previous_result,
-                                                **next_args)
-                break
-            elif text == previous_result:
-                # This result line is a repeat. Skip it.
-                continue
-            else:
-                previous_result = text
+            for line in lines:
+                text = line.decode()
 
-            try:
-                data = CdxRecord(*text.split(' '), '', '')
-                if data.status_code == '-':
-                    # the status code given for a revisit record
-                    status_code = None
+                # The resume key is delineated by a blank line.
+                if text == '':
+                    next_query = {**query, 'resumeKey': next(lines).decode()}
+                    break
+                elif text == previous_result:
+                    # This result line is a repeat. Skip it.
+                    continue
                 else:
-                    status_code = int(data.status_code)
-                length = None if data.length == '-' else int(data.length)
-                capture_time = _utils.parse_timestamp(data.timestamp)
-            except Exception as err:
-                if 'RobotAccessControlException' in text:
-                    raise BlockedByRobotsError(query["url"])
-                raise UnexpectedResponseFormat(
-                    f'Could not parse CDX output: "{text}" (query: {final_query})') from err
+                    previous_result = text
 
-            clean_url = REDUNDANT_HTTPS_PORT.sub(
-                r'\1\2', REDUNDANT_HTTP_PORT.sub(
-                    r'\1\2', data.url))
-            if skip_malformed_results and is_malformed_url(clean_url):
-                continue
-            if clean_url != data.url:
-                data = data._replace(url=clean_url)
+                try:
+                    data = CdxRecord(*text.split(' '), '', '')
+                    if data.status_code == '-':
+                        # the status code given for a revisit record
+                        status_code = None
+                    else:
+                        status_code = int(data.status_code)
+                    length = None if data.length == '-' else int(data.length)
+                    capture_time = _utils.parse_timestamp(data.timestamp)
+                except Exception as err:
+                    if 'RobotAccessControlException' in text:
+                        raise BlockedByRobotsError(query["url"])
+                    raise UnexpectedResponseFormat(
+                        f'Could not parse CDX output: "{text}" (query: {sent_query})') from err
 
-            # TODO: repeat captures have a status code of `-` and a mime type
-            # of `warc/revisit`. These can only be resolved by requesting the
-            # content and following redirects. Maybe nice to do so
-            # automatically here.
-            data = data._replace(
-                status_code=status_code,
-                length=length,
-                timestamp=capture_time,
-                raw_url=ARCHIVE_URL_TEMPLATE.format(
-                    url=data.url,
-                    timestamp=data.timestamp,
-                    mode=Mode.original.value),
-                view_url=ARCHIVE_URL_TEMPLATE.format(
-                    url=data.url,
-                    timestamp=data.timestamp,
-                    mode=Mode.view.value)
-            )
-            count += 1
-            yield data
+                clean_url = REDUNDANT_HTTPS_PORT.sub(
+                    r'\1\2', REDUNDANT_HTTP_PORT.sub(
+                        r'\1\2', data.url))
+                if skip_malformed_results and is_malformed_url(clean_url):
+                    continue
+                if clean_url != data.url:
+                    data = data._replace(url=clean_url)
+
+                # TODO: repeat captures have a status code of `-` and a mime type
+                # of `warc/revisit`. These can only be resolved by requesting the
+                # content and following redirects. Maybe nice to do so
+                # automatically here.
+                data = data._replace(
+                    status_code=status_code,
+                    length=length,
+                    timestamp=capture_time,
+                    raw_url=ARCHIVE_URL_TEMPLATE.format(
+                        url=data.url,
+                        timestamp=data.timestamp,
+                        mode=Mode.original.value),
+                    view_url=ARCHIVE_URL_TEMPLATE.format(
+                        url=data.url,
+                        timestamp=data.timestamp,
+                        mode=Mode.view.value)
+                )
+                count += 1
+                yield data
 
         return count
 
-    def get_memento(self, url, datetime=None, mode=Mode.original, *,
+    def get_memento(self, url, timestamp=None, mode=Mode.original, *,
                     exact=True, exact_redirects=None,
-                    target_window=24 * 60 * 60, follow_redirects=True):
+                    target_window=24 * 60 * 60, follow_redirects=True,
+                    # Deprecated Parameters
+                    datetime=None):
         """
         Fetch a memento (an archived HTTP response) from the Wayback Machine.
 
@@ -596,13 +662,13 @@ class WaybackClient(_utils.DepthCountedContext):
             URL to retrieve a memento of. This can be any of:
 
             - A normal URL (e.g. ``http://www.noaa.gov/``). When using this
-              form, you must also specify ``datetime``.
+              form, you must also specify ``timestamp``.
             - A ``CdxRecord`` retrieved from
               :meth:`wayback.WaybackClient.search`.
             - A URL of the memento in Wayback, e.g.
-              ``http://web.archive.org/web/20180816111911id_/http://www.noaa.gov/``
+              ``https://web.archive.org/web/20180816111911id_/http://www.noaa.gov/``
 
-        datetime : datetime.datetime or datetime.date or str, optional
+        timestamp : datetime.datetime or datetime.date or str, optional
             The time at which to retrieve a memento of ``url``. If ``url`` is
             a :class:`wayback.CdxRecord` or full memento URL, this parameter
             can be omitted.
@@ -613,7 +679,7 @@ class WaybackClient(_utils.DepthCountedContext):
             for a description of possible values.
 
             For more details, see:
-            http://archive-access.sourceforge.net/projects/wayback/administrator_manual.html#Archival_URL_Replay_Mode
+            https://archive-access.sourceforge.net/projects/wayback/administrator_manual.html#Archival_URL_Replay_Mode
 
         exact : boolean, default: True
             If false and the requested memento either doesn't exist or can't be
@@ -646,12 +712,18 @@ class WaybackClient(_utils.DepthCountedContext):
 
         Returns
         -------
-        dict : requests.Response
-            An HTTP response with the content of the memento, including a
-            history of any redirects involved. (For a complete history of all
-            HTTP requests needed to obtain the memento [rather than historic
-            redirects], check ``debug_history`` instead of ``history``.)
+        Memento
+            A :class:`Memento` object with information about the archived HTTP
+            response.
         """
+        if datetime:
+            warn('The `datetime` parameter for get_memento() was renamed to '
+                 '`timestamp`. Support for the old name will be removed '
+                 'in wayback v0.5.0; please update your code.',
+                 DeprecationWarning,
+                 stacklevel=2)
+            timestamp = timestamp or datetime
+
         if exact_redirects is None:
             exact_redirects = exact
 
@@ -677,18 +749,19 @@ class WaybackClient(_utils.DepthCountedContext):
                 original_url, original_date, mode = _utils.memento_url_data(url)
             except ValueError:
                 original_url = url
-                if not datetime:
-                    raise TypeError('You must specify `datetime` when using a '
+                if not timestamp:
+                    raise TypeError('You must specify `timestamp` when using a '
                                     'normal URL for get_memento()')
                 else:
-                    original_date = _utils.ensure_utc_datetime(datetime)
+                    original_date = _utils.ensure_utc_datetime(timestamp)
 
         original_date_wayback = _utils.format_timestamp(original_date)
         url = ARCHIVE_URL_TEMPLATE.format(timestamp=original_date_wayback,
                                           mode=mode,
                                           url=original_url)
 
-        with _utils.rate_limited(calls_per_second=30, group='get_memento'):
+        with _utils.rate_limited(calls_per_second=self.session.memento_calls_per_second,
+                                 group='get_memento'):
             # Correctly following redirects is actually pretty complicated. In
             # the simplest case, a memento is a simple web page, and that's
             # no problem. However...
