@@ -1,5 +1,6 @@
 from datetime import date, datetime, timezone, timedelta
 from pathlib import Path
+import time
 import pytest
 import vcr
 import requests
@@ -150,19 +151,6 @@ def test_search_raises_for_blocked_urls():
             next(versions)
 
 
-@pytest.mark.parametrize('parameter', ('output', 'fl', 'showDupeCount',
-                                       'showSkipCount', 'lastSkipTimestamp',
-                                       'showNumPages', 'showPagedIndex'))
-def test_search_raises_for_unsupported_parameters(parameter):
-    with pytest.raises(ValueError) as excinfo:
-        with WaybackClient() as client:
-            versions = client.search(**{'url': 'https://energystar.gov/',
-                                        parameter: 'arbitrary_value'})
-            next(versions)
-
-    assert parameter in str(excinfo.value)
-
-
 def test_search_removes_malformed_entries(requests_mock):
     """
     The CDX index contains many lines for things that can't actually be
@@ -178,7 +166,7 @@ def test_search_removes_malformed_entries(requests_mock):
         bad_cdx_data = f.read()
 
     with WaybackClient() as client:
-        requests_mock.get('http://web.archive.org/cdx/search/cdx'
+        requests_mock.get('https://web.archive.org/cdx/search/cdx'
                           '?url=https%3A%2F%2Fepa.gov%2F%2A'
                           '&from=20200418000000&to=20200419000000'
                           '&showResumeKey=true&resolveRevisits=true',
@@ -202,13 +190,13 @@ def test_search_handles_no_length_cdx_records(requests_mock):
         bad_cdx_data = f.read()
 
     with WaybackClient() as client:
-        requests_mock.get('http://web.archive.org/cdx/search/cdx'
+        requests_mock.get('https://web.archive.org/cdx/search/cdx'
                           '?url=www.cnn.com%2F%2A'
                           '&matchType=domain&filter=statuscode%3A200'
                           '&showResumeKey=true&resolveRevisits=true',
                           [{'status_code': 200, 'text': bad_cdx_data}])
         records = client.search('www.cnn.com/*',
-                                matchType="domain",
+                                match_type="domain",
                                 filter_field="statuscode:200")
 
         record_list = list(records)
@@ -230,13 +218,13 @@ def test_search_handles_bad_timestamp_cdx_records(requests_mock):
         bad_cdx_data = f.read()
 
     with WaybackClient() as client:
-        requests_mock.get('http://web.archive.org/cdx/search/cdx'
+        requests_mock.get('https://web.archive.org/cdx/search/cdx'
                           '?url=www.usatoday.com%2F%2A'
                           '&matchType=domain&filter=statuscode%3A200'
                           '&showResumeKey=true&resolveRevisits=true',
                           [{'status_code': 200, 'text': bad_cdx_data}])
         records = client.search('www.usatoday.com/*',
-                                matchType="domain",
+                                match_type="domain",
                                 filter_field="statuscode:200")
 
         record_list = list(records)
@@ -253,17 +241,23 @@ def test_search_handles_bad_timestamp_cdx_records(requests_mock):
 def test_get_memento():
     with WaybackClient() as client:
         memento = client.get_memento('https://www.fws.gov/birds/',
-                                     datetime=datetime(2017, 11, 24, 15, 13, 15))
+                                     timestamp=datetime(2017, 11, 24, 15, 13, 15))
         assert 'https://www.fws.gov/birds/' == memento.url
         assert datetime(2017, 11, 24, 15, 13, 15, tzinfo=timezone.utc) == memento.timestamp
         assert 'id_' == memento.mode
+        assert memento.headers == {
+            'Content-Type': 'text/html',
+            'Date': 'Fri, 24 Nov 2017 15:13:14 GMT',
+            'Strict-Transport-Security': 'max-age=31536000; includeSubDomains; preload',
+            'Transfer-Encoding': 'chunked'
+        }
 
 
 @ia_vcr.use_cassette()
 def test_get_memento_with_date_datetime():
     with WaybackClient() as client:
         memento = client.get_memento('https://www.fws.gov/birds/',
-                                     datetime=date(2017, 11, 24),
+                                     timestamp=date(2017, 11, 24),
                                      exact=False)
         assert 'https://www.fws.gov/birds/' == memento.url
         assert datetime(2017, 11, 24, 15, 13, 15, tzinfo=timezone.utc) == memento.timestamp
@@ -274,7 +268,7 @@ def test_get_memento_with_date_datetime():
 def test_get_memento_with_string_datetime():
     with WaybackClient() as client:
         memento = client.get_memento('https://www.fws.gov/birds/',
-                                     datetime='20171124151315')
+                                     timestamp='20171124151315')
         assert 'https://www.fws.gov/birds/' == memento.url
         assert datetime(2017, 11, 24, 15, 13, 15, tzinfo=timezone.utc) == memento.timestamp
         assert 'id_' == memento.mode
@@ -284,7 +278,7 @@ def test_get_memento_with_string_datetime():
 def test_get_memento_with_inexact_string_datetime():
     with WaybackClient() as client:
         memento = client.get_memento('https://www.fws.gov/birds/',
-                                     datetime='20171124151310',
+                                     timestamp='20171124151310',
                                      exact=False)
         assert 'https://www.fws.gov/birds/' == memento.url
         assert datetime(2017, 11, 24, 15, 13, 15, tzinfo=timezone.utc) == memento.timestamp
@@ -298,7 +292,7 @@ def test_get_memento_handles_non_utc_datetime():
         requested_time = datetime(2017, 11, 24, 8, 13, 15,
                                   tzinfo=timezone(timedelta(hours=-7)))
         memento = client.get_memento('https://www.fws.gov/birds/',
-                                     datetime=requested_time)
+                                     timestamp=requested_time)
 
         assert 'https://www.fws.gov/birds/' == memento.url
         assert datetime(2017, 11, 24, 15, 13, 15, tzinfo=timezone.utc) == memento.timestamp
@@ -310,7 +304,7 @@ def test_get_memento_with_invalid_datetime_type():
     with WaybackClient() as client:
         with pytest.raises(TypeError):
             client.get_memento('https://www.fws.gov/birds/',
-                               datetime=True)
+                               timestamp=True)
 
 
 @ia_vcr.use_cassette()
@@ -324,13 +318,13 @@ def test_get_memento_with_requires_datetime_with_regular_url():
 def test_get_memento_with_archive_url():
     with WaybackClient() as client:
         memento = client.get_memento(
-            'http://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/')
+            'https://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/')
 
         # Metadata About the Memento
         assert 'https://www.fws.gov/birds/' == memento.url
         assert datetime(2017, 11, 24, 15, 13, 15, tzinfo=timezone.utc) == memento.timestamp
         assert 'id_' == memento.mode
-        assert 'http://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/' == memento.memento_url
+        assert 'https://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/' == memento.memento_url
         assert () == memento.history
         assert () == memento.debug_history
 
@@ -359,8 +353,8 @@ def test_get_memento_with_cdx_record():
                            200,
                            'abc',
                            100,
-                           'http://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/',
-                           'http://web.archive.org/web/20171124151315/https://www.fws.gov/birds/')
+                           'https://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/',
+                           'https://web.archive.org/web/20171124151315/https://www.fws.gov/birds/')
         memento = client.get_memento(record)
         assert 'https://www.fws.gov/birds/' == memento.url
         assert datetime(2017, 11, 24, 15, 13, 15, tzinfo=timezone.utc) == memento.timestamp
@@ -371,25 +365,25 @@ def test_get_memento_with_cdx_record():
 def test_get_memento_with_mode():
     with WaybackClient() as client:
         memento = client.get_memento('https://www.fws.gov/birds/',
-                                     datetime=datetime(2017, 11, 24, 15, 13, 15),
+                                     timestamp=datetime(2017, 11, 24, 15, 13, 15),
                                      mode=Mode.view)
         assert '' == memento.mode
-        assert 'http://web.archive.org/web/20171124151315/https://www.fws.gov/birds/' == memento.memento_url
+        assert 'https://web.archive.org/web/20171124151315/https://www.fws.gov/birds/' == memento.memento_url
 
         memento = client.get_memento('https://www.fws.gov/birds/',
-                                     datetime=datetime(2017, 11, 24, 15, 13, 15))
+                                     timestamp=datetime(2017, 11, 24, 15, 13, 15))
         assert 'id_' == memento.mode
-        assert 'http://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/' == memento.memento_url
+        assert 'https://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/' == memento.memento_url
 
 
 @ia_vcr.use_cassette()
 def test_get_memento_with_mode_string():
     with WaybackClient() as client:
         memento = client.get_memento('https://www.fws.gov/birds/',
-                                     datetime=datetime(2017, 11, 24, 15, 13, 15),
+                                     timestamp=datetime(2017, 11, 24, 15, 13, 15),
                                      mode='id_')
         assert 'id_' == memento.mode
-        assert 'http://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/' == memento.memento_url
+        assert 'https://web.archive.org/web/20171124151315id_/https://www.fws.gov/birds/' == memento.memento_url
 
 
 @ia_vcr.use_cassette()
@@ -397,7 +391,7 @@ def test_get_memento_with_mode_boolean_is_not_allowed():
     with WaybackClient() as client:
         with pytest.raises(TypeError):
             client.get_memento('https://www.fws.gov/birds/',
-                               datetime=datetime(2017, 11, 24, 15, 13, 15),
+                               timestamp=datetime(2017, 11, 24, 15, 13, 15),
                                mode=True)
 
 
@@ -425,7 +419,7 @@ def test_get_memento_raises_when_memento_is_outside_target_window():
 def test_get_memento_with_redirects():
     with WaybackClient() as client:
         memento = client.get_memento(
-            'http://web.archive.org/web/20180808094144id_/https://www.epa.gov/ghgreporting/san5779-factsheet')
+            'https://web.archive.org/web/20180808094144id_/https://www.epa.gov/ghgreporting/san5779-factsheet')
         assert len(memento.history) == 1        # memento redirects
         assert len(memento.debug_history) == 2  # actual HTTP redirects
 
@@ -434,7 +428,7 @@ def test_get_memento_with_redirects():
 def test_get_memento_with_path_based_redirects():
     """
     Most redirects in Wayback redirect to a complete URL, with headers like:
-        Location: http://web.archive.org/web/20201027215555id_/https://www.whitehouse.gov/administration
+        Location: https://web.archive.org/web/20201027215555id_/https://www.whitehouse.gov/administration
     But some include only an absolute path, e.g:
         Location: /web/20201027215555id_/https://www.whitehouse.gov/ostp/about/student/faqs
     This tests that we correctly handle the latter situation.
@@ -450,7 +444,7 @@ def test_get_memento_with_path_based_redirects():
 def test_get_memento_with_schemeless_redirects():
     """
     Most redirects in Wayback redirect to a complete URL, with headers like:
-        Location: http://web.archive.org/web/20201027215555id_/https://www.whitehouse.gov/administration
+        Location: https://web.archive.org/web/20201027215555id_/https://www.whitehouse.gov/administration
     But some do not include a scheme:
         Location: //web.archive.org/web/20201102232816id_/https://www.census.gov/geo/gssi/
     This tests that we correctly handle the latter situation.
@@ -500,16 +494,16 @@ def test_get_memento_follows_historical_redirects():
         #   https://www.epa.gov/sites/production/files/signpost/cc.html
         #
         # What should happen here under the hood:
-        # http://web.archive.org/web/20200201020357id_/http://epa.gov/climatechange
+        # https://web.archive.org/web/20200201020357id_/http://epa.gov/climatechange
         #   Is not a memento, and sends us to:
-        #   http://web.archive.org/web/20200201023757id_/https://www.epa.gov/climatechange
+        #   https://web.archive.org/web/20200201023757id_/https://www.epa.gov/climatechange
         #     Which is a memento of a redirect to:
-        #     http://web.archive.org/web/20200201023757id_/https://www.epa.gov/sites/production/files/signpost/cc.html
+        #     https://web.archive.org/web/20200201023757id_/https://www.epa.gov/sites/production/files/signpost/cc.html
         #       ...which is not a memento, and redirects to:
-        #       http://web.archive.org/web/20200201024405id_/https://www.epa.gov/sites/production/files/signpost/cc.html
-        start_url = ('http://web.archive.org/web/20200201020357id_/'
+        #       https://web.archive.org/web/20200201024405id_/https://www.epa.gov/sites/production/files/signpost/cc.html
+        start_url = ('https://web.archive.org/web/20200201020357id_/'
                      'http://epa.gov/climatechange')
-        target = ('http://web.archive.org/web/20200201024405id_/'
+        target = ('https://web.archive.org/web/20200201024405id_/'
                   'https://www.epa.gov/sites/production/files/signpost/cc.html')
         memento = client.get_memento(start_url, exact=False)
         assert 'https://www.epa.gov/sites/production/files/signpost/cc.html' == memento.url
@@ -525,17 +519,17 @@ def test_get_memento_follow_redirects_does_not_follow_historical_redirects():
         #   https://www.epa.gov/sites/production/files/signpost/cc.html
         #
         # What should happen here under the hood:
-        # http://web.archive.org/web/20200201020357id_/http://epa.gov/climatechange
+        # https://web.archive.org/web/20200201020357id_/http://epa.gov/climatechange
         #   Is not a memento, and sends us to:
-        #   http://web.archive.org/web/20200201023757id_/https://www.epa.gov/climatechange
+        #   https://web.archive.org/web/20200201023757id_/https://www.epa.gov/climatechange
         #     Which is a memento of a redirect. Because follow_redirects=False,
         #     we should *not* follow it to:
-        #     http://web.archive.org/web/20200201023757id_/https://www.epa.gov/sites/production/files/signpost/cc.html
+        #     https://web.archive.org/web/20200201023757id_/https://www.epa.gov/sites/production/files/signpost/cc.html
         #       ...and then to:
-        #       http://web.archive.org/web/20200201024405id_/https://www.epa.gov/sites/production/files/signpost/cc.html
-        start_url = ('http://web.archive.org/web/20200201020357id_/'
+        #       https://web.archive.org/web/20200201024405id_/https://www.epa.gov/sites/production/files/signpost/cc.html
+        start_url = ('https://web.archive.org/web/20200201020357id_/'
                      'http://epa.gov/climatechange')
-        target = ('http://web.archive.org/web/20200201023757id_/'
+        target = ('https://web.archive.org/web/20200201023757id_/'
                   'https://www.epa.gov/climatechange')
         memento = client.get_memento(start_url, exact=False, follow_redirects=False)
         assert 'https://www.epa.gov/climatechange' == memento.url
@@ -636,3 +630,73 @@ class TestWaybackSession:
         # Overwriting the default
         res = session.request('GET', 'http://test.com', timeout=1)
         assert res.text == '1'
+
+    @ia_vcr.use_cassette()
+    def test_search_rate_limits(self):
+        # The timing relies on the cassettes being present,
+        # therefore the first run might fail.
+        # Since another test might run before this one,
+        # I have to wait one second before starting.
+        time.sleep(1)
+        # First test that the default rate limits are correctly applied.
+        start_time = time.time()
+        with WaybackClient() as client:
+            for i in range(3):
+                next(client.search('zew.de'))
+        duration_with_limits = time.time() - start_time
+
+        # Check that disabling the rate limits through the search API works.
+        start_time = time.time()
+        with WaybackClient(WaybackSession(search_calls_per_second=0)) as client:
+            for i in range(3):
+                next(client.search('zew.de'))
+        duration_without_limits = time.time() - start_time
+
+        # Check that a different rate limit set through the session is applied correctly.
+        # I need to sleep one half second in order to reset the rate limit.
+        time.sleep(0.5)
+        start_time = time.time()
+        with WaybackClient(WaybackSession(search_calls_per_second=2)) as client:
+            for i in range(3):
+                next(client.search('zew.de'))
+        duration_with_limits_custom = time.time() - start_time
+
+        assert 1.3 <= duration_with_limits <= 1.4
+        assert 0.0 <= duration_without_limits <= 0.05
+        assert 0.0 <= duration_with_limits_custom <= 1.05
+
+    @ia_vcr.use_cassette()
+    def test_memento_rate_limits(self):
+        # The timing relies on the cassettes being present,
+        # therefore the first run might fail.
+        # Since another test might run before this one,
+        # I have to wait one call before starting.
+        time.sleep(1/30)
+        with WaybackClient() as client:
+            cdx = next(client.search('zew.de'))
+        # First test that the default rate limits are correctly applied.
+        start_time = time.time()
+        with WaybackClient() as client:
+            for i in range(11):
+                client.get_memento(cdx)
+        duration_with_limits = time.time() - start_time
+
+        # Check that disabling the rate limits through the get_memento API works.
+        start_time = time.time()
+        with WaybackClient(WaybackSession(memento_calls_per_second=0)) as client:
+            for i in range(3):
+                client.get_memento(cdx)
+        duration_without_limits = time.time() - start_time
+
+        # Check that a different rate limit set through the session is applied correctly.
+        # I need to sleep 0.1 seconds in order to reset the rate limit.
+        time.sleep(0.1)
+        start_time = time.time()
+        with WaybackClient(WaybackSession(memento_calls_per_second=10)) as client:
+            for i in range(6):
+                client.get_memento(cdx)
+        duration_with_limits_custom = time.time() - start_time
+
+        assert 0.33 <= duration_with_limits <= 0.43
+        assert 0.0 <= duration_without_limits <= 0.05
+        assert 0.5 <= duration_with_limits_custom <= 0.55
